'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.wareBuilder = wareBuilder;

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function flatten(arg) {
  return arg.reduce(function (memo, item) {
    return Array.isArray(item) ? [].concat(_toConsumableArray(memo), _toConsumableArray(flatten(item))) : [].concat(_toConsumableArray(memo), [item]);
  }, []);
}

function normalizeArgumentArray(args) {
  return flatten(args).filter(function (item) {
    return !!item;
  });
}

function isFunction(fn) {
  return typeof fn === 'function';
}

function isPromise(obj) {
  return !!obj && ((typeof obj === 'undefined' ? 'undefined' : _typeof(obj)) === 'object' || typeof obj === 'function') && typeof obj.then === 'function';
}

function wareBuilder() {
  // high level - returns a function that can merge results
  return function middle() {
    for (var _len = arguments.length, middleware = Array(_len), _key = 0; _key < _len; _key++) {
      middleware[_key] = arguments[_key];
    }

    // this is the actual warewolf() call
    var middlewareQueue = normalizeArgumentArray(middleware);
    var hasCallback = true;
    var isDone = false;
    return function () {
      for (var _len2 = arguments.length, invocationArguments = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
        invocationArguments[_key2] = arguments[_key2];
      }

      // some assumptions here about how warewolf is called
      // if callback is provided, that's subbed with next() in middleware
      // if no callback, we treat it like a promise
      var arity = invocationArguments.length;
      var isErrorMethod = function isErrorMethod(fn) {
        return fn.length === arity + 1;
      };

      var done = invocationArguments.pop();
      if (!isFunction(done)) {
        if (done) {
          invocationArguments.push(done);
        }
        hasCallback = false;
      }

      var promiseBody = function promiseBody(resolve, reject) {
        if (!hasCallback) {
          done = function done(err, result) {
            if (err) {
              reject(err);
            } else {
              resolve(result);
            }
          };
        }

        function moveToNext(err) {
          while (middlewareQueue.length && (err ? !isErrorMethod(middlewareQueue[0]) : isErrorMethod(middlewareQueue[0]))) {
            middlewareQueue.shift();
          }
          if (middlewareQueue.length) {
            var fn = middlewareQueue.shift();
            return err ? fn.bind(this, err) : fn;
          }
          return null;
        }

        var nextStep = function nextStep(err) {
          for (var _len3 = arguments.length, args = Array(_len3 > 1 ? _len3 - 1 : 0), _key3 = 1; _key3 < _len3; _key3++) {
            args[_key3 - 1] = arguments[_key3];
          }

          var nextFn = moveToNext(err);
          if (!nextFn) {
            isDone = true;
            return done.apply(undefined, [err].concat(args));
          }
          var flowArgs = hasCallback ? [nextStep] : [];

          var result = nextFn.apply(undefined, invocationArguments.concat(flowArgs));
          if (isPromise(result)) {
            result.then(function (r) {
              return nextStep(null, r);
            }).catch(nextStep);
          } else if (!hasCallback) {
            return nextStep(err, result);
          }
          return result;
        };

        try {
          return nextStep();
        } catch (e) {
          if (e.name === 'AssertionError') {
            throw e;
          }
          if (middlewareQueue.find(isErrorMethod)) {
            return nextStep(e);
          }
          if (!isDone) {
            return done(e);
          }
          throw e;
        }
      };

      if (!hasCallback) {
        return new Promise(promiseBody);
      }
      return promiseBody();
    };
  };
}

exports.default = wareBuilder();